package company;

import java.util.LinkedList;

/**
 * Maintains top level information about the company structure
 */
public class Company {

    /*
     * State variables
     */
    private Employee _ceo;
    private LinkedList<Department> _departments = new LinkedList<Department>();
    private String _name;

    /** Add new department */
    public void addDepartment(Department deptm) {
        _departments.add(deptm);
    }

    /** Appoint the CEO of the company */
    public void appointCEO(Employee empl) {
        _ceo = empl;
        _ceo.setRole(Employee.CEO);
        System.out.println("Welcome new big boss " + _ceo.getFullName() + "!");
    }

    /** Get current CEO */
    public Employee getCEO() {
        return _ceo;
    }
    

    /** Exchange the roles of two employees */
    public void exchangeRoles(Employee empl1, Employee empl2) {

        System.out.println("Exchanging " + empl1.getFullName() + " and "
                + empl2.getFullName());

        Department deptm1 = empl1.getDepartment();
        int role1 = empl1.getRole();

        Department deptm2 = empl2.getDepartment();
        int role2 = empl2.getRole();

        transferTo(empl1, role2, deptm2);
        transferTo(empl2, role1, deptm1);
    }

    /** Get department at given index */
    public Department getDepartmentAt(int i1) {
        return (Department) _departments.get(i1);
    }

    /** Get the number of departments in the company */
    public int getDepartmentCount() {
        return _departments.size();
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
        return _name;
    }
    
    /**
     * @param name The name to set.
     */
    public void setName(String name) {
        _name = name;
    }
    
    /** Transfer employee to another position in the company */
    public void transferTo(Employee empl, int role, Department deptm) {
        if (empl.getDepartment() != null) {
            empl.leaveDepartment();
        }

        switch (role) {
        case Employee.CEO:
            appointCEO(empl);
            break;

        case Employee.DEPARTMENT_MGR:
            deptm.appointManager(empl);
            break;

        default:
            empl.setRole(Employee.WORKER);
            empl.assignTo(deptm);
            break;
        }
    }
}