package company;

import java.util.LinkedList;

/**
 * Maintains information about a department
 */
public class Department {
	
	/* State variables */
	private String _name;

	private Employee _manager = null;
	
	// lazy programmer made list public instead of providing access methods:
	public LinkedList<Employee> _workers = new LinkedList<Employee>();
	
	/** Initialize department */
	public Department(String name) {
		_name = name;
	}
	
	/** Get department name */
	public String getName() {
		return _name;
	}		
	
	/** Get department manager */
	public Employee getManager() {
		return _manager;
	}
	
	/** Register employee as a worker of the department */
	public void registerWorker(Employee empl) {
		_workers.add(empl);
	}
	
	/** Unregister employee from the department */
	public void unregisterWorker(Employee empl) {
		_workers.remove(empl);
	}
	
	/** Appoint the manager of the department */
	public void appointManager(Employee empl) {
		_manager = empl;
		
		if (_manager.getDepartment() != this) {
			_manager.assignTo(this);
		}
		
		_manager.setRole(Employee.DEPARTMENT_MGR);
	}
}
