package company;

/**
 * Manages information about an employee of the company
 */
public class Employee {
	
	/*
	 * Sex constants
	 */
	static final public int FEMALE = 0;
	static final public int MALE = 1;
	
	/*
	 * Role constants
	 */
	static final public int WORKER = 0;
	static final public int DEPARTMENT_MGR = 1;
	static final public int CEO = 2;
	
	/*
	 * State variables
	 */
	private String[] _name = new String[0];
	
	private int _sex = -1;
	
	private int _role = -1;
	
	private Department _departm = null;
	
	/** Initialize Emploee */
	public Employee(String firstName, String familyName, int sex) {
		_name = new String[2];
		_name[0] = firstName;
		_name[1] = familyName;
		_sex = sex;
		_role = WORKER;
	}
	
	/** Get formatted full name of the employee */
	public String getFullName() {
		String fullName = getSex() == Employee.FEMALE ? "Ms." : "Mr.";
		
		for (int i1 = 0; i1 < _name.length; i1++) {
			fullName += " " + _name[i1];
		}
		return fullName;
	}
	
	/** Get initials of the employee */
	public String getInitials() {
		String initials = "";
		
		for (int i1 = 0; i1 < _name.length; i1++) {
			initials += _name[i1].charAt(0);
		}
		return initials;
	}
	
	/** Get employee sex (see constants) */
	public int getSex() {
		return _sex;
	}
	
	/** Get employee role in the company (see constants) */
	public int getRole() {
		return _role;
	}
	
	/** Get employee deparment (can be null) */
	public Department getDepartment() {
		return _departm;
	}
	
	/** Set employee role in the company */
	public void setRole(int role) {
		_role = role;
	}
	
	/** Assign employee to a department */
	public void assignTo(Department departm) {
		_departm = departm;
		_departm.registerWorker(this);
	}
	
	/** Leave current department */
	public void leaveDepartment() {
		_departm.unregisterWorker(this);
		_departm = null;
	}
}
