package companydisplay;

import hierarchydisplay.HierarchyDisplayImpl;

import java.util.LinkedList;

import base company.Company;
import base company.Department;
import base company.Employee;

/**
 * Binding to display company management hierarchy
 */
public team class CompanyHierarchyDisplay extends HierarchyDisplayImpl {
    
    /* State variables */
    protected CompanyNode _company = null;
    
    
    /** Set company */
    public CompanyHierarchyDisplay(Company as CompanyNode comp) {
    	activate(ALL_THREADS);
        _company = comp;
    }
    
    /** Get root node of the hierarchy */
    public Node getRoot() {
        return _company;
    }

    public class WorkerNode extends Node playedBy Employee {

        /** Get node name to be displayed */
        public String getText() {
            String name = getFullName();
            if (textFits(name)) {
                return name;
            } else {
                return getInitials();
            }
        }
        
        // provide details for getText(): 
        String getFullName()  -> String getFullName();
        String getInitials()  -> String getInitials(); 
    }
    
    public class CompanyNode extends CompositeNode playedBy Company {
        
        /** Get node text to be displayed */
        public String getText() {
            String name = getName();
            if (textFits(name + " (CEO)")) {
                return name + " (CEO)";
            } else if (textFits(name)) {
                return name;
            } else {
                return getInitials();
            }
        }
        
        // provide details for getText():
        String getInitials() -> Employee getCEO() 
        with {
            result <- result.getInitials() 
        }
        
        String getName() -> Employee getCEO() 
        with
        {
            result <- result.getFullName()
        }
                
        
        // provide implementation for implicitly inherited abstract methods:
        getChildCount                 -> getDepartmentCount; 

        public Node getChildAt(int i1) {
            return getChild(i1);
        }

        // New method returning more specific type DepartmentNode instead of Node.
        // This is needed because Node has no playedBy and thus cannot be used for lifting.
        DepartmentNode getChild(int i)-> Department getDepartmentAt(int i);

        
        // install updating mechanism:
        void recalculateLayout() <- after void addDepartment(Department d),
                                          void exchangeRoles(Employee e1, Employee e2);
        
        void recalculateLayoutAfterTransfer(Object e, int role, Object d)
                                 <- after void transferTo(Employee e, int role, Department d)
                when (d != null && transferComplete());
        
        /**
         * Assures that the callin binding is only effective after the transfer is
         * complete.
         */
        private boolean transferComplete() {
            for(int i = 0; i < getChildCount(); i++)
                if(getChild(i).getManager() == null) return false;
            return true;
        }
        
        private void recalculateLayoutAfterTransfer(Object e, int role, Object d) {
            CompanyHierarchyDisplay.this.calculateLayout();
        }
        
        protected void recalculateLayout() {
            CompanyHierarchyDisplay.this.calculateLayout();
        }
    }
    
    /***************************************************************************
     * Department manager node
     */
    public class DepartmentNode extends CompositeNode playedBy Department {
        
        /** Get node text to be displayed */
        public String getText() {
            String mgrName = getManager().getFullName(); 
            String deptmName = " (" + getName() + ")";
            if (textFits(mgrName + deptmName)) {
                return mgrName + deptmName;
            } else if (textFits(mgrName)) {
                return mgrName;
            } else {
                return getManager().getInitials();
            }
        }
        
        WorkerNode getManager()       -> Employee getManager();
        String getName()              -> String getName();

        WorkerNode getWorkerAt(int i) -> get LinkedList<Employee> _workers
            with {             result <- _workers.get(i) }
        
        int getWorkerCount()          -> get LinkedList<Employee> _workers
            with {             result <- _workers.size() }
        
        protected void recalculateLayout() {
            CompanyHierarchyDisplay.this.calculateLayout();
        }
        
        /** Get child node at given index */
        public Node getChildAt(int i1) {
            if (i1 >= findManager()) {
                i1++;
            }
            return getWorkerAt(i1);
        }
        
        /** Get the number of children nodes */
        public int getChildCount() {
            /* -1 to exclude manager */
            return getWorkerCount() - 1;
        }
        
        /** Determine index of the manager in the department worker list */
        private int findManager() {
            for (int i1 = 0; i1 < getWorkerCount(); i1++)
				if (getWorkerAt(i1) == getManager())
					return i1;
            return getWorkerCount();
        }
    }
}
