package gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.UIManager;

import com.jgoodies.plaf.FontSizeHints;
import com.jgoodies.plaf.LookUtils;
import com.jgoodies.plaf.Options;
import company.Company;
import company.Database;

/**
 * Main application class: creates user interface, initializes data.
 */
public class Application 
{
	/*
	 * State variables
	 */
	private HierarchyView _view = new HierarchyView();
	private JFrame _frame = new JFrame();
	private HierarchyOptionsHandler _optionsHandler = null; 
	
	private Company _company = new Company();
	
	/** Entry point to the application */ 
	public static void main(String[] args) {
      Application app = new Application();
		app.start();
	}
    
	/** Starts the application by initializing GUI */
	public void start()	{
		Database.load(_company);
		configureUI();
		buildGraphicalInterface();
		makeBrowser();
	}
	
	/** Build window for the graphical tree display */
	private void buildGraphicalInterface() {
		_frame.setSize(750, 400);
		_frame.setContentPane(_view);
		_frame.setTitle("Object Teams / Java - Hierarchy Example: Tree View");
		_frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		_frame.setJMenuBar(buildMenuBar());
		_frame.setLocation(20, 20);
		_frame.setVisible(true);
	}
	
	/** Build application menu bar. */
	private JMenuBar buildMenuBar() {
		JMenuItem menuItem;
		JMenu menu = new JMenu("Change");
		JMenuBar menuBar = new JMenuBar();
		
		menuBar.add(menu);
		
		menuItem = new JMenuItem("Random Change");
		menuItem.addActionListener(new ChangeActionHandler(_company));
		menu.add(menuItem);
		
		_optionsHandler = new HierarchyOptionsHandler(_company, _view);
		
		menuBar.add(_optionsHandler.buildMenu());
		_optionsHandler.applyOptions();
		
		return menuBar;
    }
	
	/** Setup look & feel */
	private void configureUI()	{
		UIManager.put(Options.USE_SYSTEM_FONTS_APP_KEY, Boolean.TRUE);
		Options.setGlobalFontSizeHints(FontSizeHints.MIXED);
		Options.setDefaultIconSize(new Dimension(18, 18));

		String lafName =
			LookUtils.isWindowsXP()
				? Options.getCrossPlatformLookAndFeelClassName()
				: Options.getSystemLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(lafName);
		} 
		catch (Exception e)	{
			System.err.println("Can't set look & feel:" + e);
		}
	}
	
	/**
	 * This method initializes the tree browser window (2nd view). 
	 */
	private void makeBrowser() {
	    // create the browser:
	    CompanyTree ctree = new CompanyTree(_company);
	    
       // extract swing component and do the layout:
       JComponent view = ctree.getView();

       // most simple layout in a new frame:
       JFrame frame = new JFrame("Object Teams / Java - Hierarchy Exmaple: Browser View");
       frame.setSize(700, 210);
       Container contentPane = frame.getContentPane();
       contentPane.setLayout(new BorderLayout());
       contentPane.add("Center", new JScrollPane(view));

       // let the user close the application by closing the window:
       frame.addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent we) {
               System.exit(0);
           }
       });

       frame.setLocation(20, 430);
       // and here we go:
       frame.setVisible(true);
   }
}