/*
 * Created on 22.06.2005
 *
 */
package gui;

import tree.ColumnTree;

import javax.swing.tree.TreeNode;
import PIRL.TreeTable.TreeTableModel;
import java.util.LinkedList;

import base company.Company;
import base company.Department;
import base company.Employee;

/**
 * This team is a connector that allows to view the company hierarchy 
 * in a tree browser.
 */
public team class CompanyTree extends ColumnTree {
    
    /**
     * Lifting constructor, receives a <code>Company</code> object.
     * 
     * @param comp
     *            The company to show in the tree bowser.
     */
    public CompanyTree(Company as CompanyNode comp) {
        super(comp);
    }

    /**
     * The root node of the tree.
     */
    public class CompanyNode extends MyTreeNode playedBy Company {
        
    	// === Implement methods inherited from MyTreeNode: ===
    	
    	// define a two column display (company name -- CEO name):
        public String getValueAt(int column) {
            if (column == 0)
                return getName();
            return getCEOName(); 
        }

    	// use existing encapsulation of collection Company._departments:
        int getChildCount()           -> int getDepartmentCount();
        MyTreeNode getChildAtT(int i) -> Department getDepartmentAt(int i);

        // simple Virtual Restructuring: inline the name of a company's CEO:
        String getCEOName()           -> Employee getCEO()
        	with {             result <- result.getFullName() }
        
        String getName()              -> String getName();
        
        // === simple Notification: ===
        structureChanged               <- after transferTo;
    }

    /**
     * A node representing a department of the company.
     */
    public class DepartmentNode extends MyTreeNode playedBy Department {

    	// === Implement methods inherited from MyTreeNode: ===
    	
    	// define a two column display (department name -- manager name):
        public String getValueAt (int column) {
            switch (column) {
                case 0: return getName();
                case 1: return getManagerName();
            }
            return "";
        }

        // Virtual Restructuring for accessing collection _workers:
    	
    	int getChildCount()           -> get LinkedList<Employee> _workers 
			with {             result <- _workers.size() } 
        
    	MyTreeNode getChildAtT(int i) -> get LinkedList<Employee> _workers
			with {             result <- (Employee)_workers.get(i) }
        
        // simple Virtual Restructuring: inline the name of a department's manager:
        String getManagerName()       -> Employee getManager()
    	    with {             result <- result.getFullName()};

    	String getName()              -> String getName();

    }
    
    /**
     * A leaf node of the tree.
     */
    public class WorkerNode extends LeafNode playedBy Employee {

    	// === Implement methods inherited from MyTreeNode: ===
    	
    	// use only first of two columns:
        public String getValueAt(int i) {
            return i == 0 ? getFullName() : "";
        }
        
        String getFullName() -> String getFullName();
    }

    /**
     * The tree model defines the table layout.
     */
    public class MyTreeModel {
        
        /**
         * @param root
         *            The root node.
         */
        public MyTreeModel(TreeNode root) {
            tsuper(
                root,
                new String[]{" ", "CEO / Manager"},
                new Class[] {TreeTableModel.class, String.class});
        }
    }
 }