package gui;

import hierarchydisplay.VersionA;
import hierarchydisplay.VersionB;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;

import company.Company;
import companydisplay.CompanyHierarchyDisplay;

/**
 * Handle user options for hierarchy view
 */
public class HierarchyOptionsHandler implements ActionListener {
	
	/*
	 * data from context
	 */
    private HierarchyView _view;	
    private CompanyHierarchyDisplay _hier; 
    private Company _company;
   
    private VersionA _versionA;
    private VersionB _versionB; 
   
	/*
	 * options
	 */
	private boolean isRightAngled = false;
	private boolean isAdjusted = false;
	
	/*
	 * menu items
	 */
	JCheckBoxMenuItem menuStraight = null;
	JCheckBoxMenuItem menuRightAngled = null;
	JCheckBoxMenuItem menuIdentical = null;
	JCheckBoxMenuItem menuAdjusted = null;	
	JCheckBoxMenuItem menuCompany = null;
	JCheckBoxMenuItem menuAWT = null;	
	
	/** Construction */
	public HierarchyOptionsHandler(Company company, HierarchyView view) {
		_company = company;
		_view = view;
      _hier = new CompanyHierarchyDisplay(company);
      // prepare adaptations for this hierarchy display:
      _versionA = new VersionA(_hier);
      _versionB = new VersionB(_hier);
	}
	
	/** Build options menu */
	public JMenu buildMenu() {
		JMenu menu = new JMenu("Options");
		
		menuStraight = new JCheckBoxMenuItem("Straight connectors");
		menuStraight.addActionListener(this);
		menu.add(menuStraight);
		
		menuRightAngled = new JCheckBoxMenuItem("Rightangled connectors");
		menuRightAngled.addActionListener(this);
		menu.add(menuRightAngled);
		
		menuIdentical = new JCheckBoxMenuItem("Identical nodes");
		menuIdentical.addActionListener(this);
		menu.add(menuIdentical);
		
		menuAdjusted = new JCheckBoxMenuItem("Adjusted nodes");
		menuAdjusted.addActionListener(this);
		menu.add(menuAdjusted);
		
		return menu;
    }
	
	/** Handle menu items */
	public void actionPerformed(ActionEvent e) {
    	String actionCmd = e.getActionCommand();
    	if (actionCmd.startsWith("Straight")) {
    		isRightAngled = false;
    	}
    	else if (actionCmd.startsWith("Rightangled")) {
    		isRightAngled = true;
    	}
    	else if (actionCmd.startsWith("Identical")) {
    		isAdjusted = false;
    	}
    	else if (actionCmd.startsWith("Adjusted")) {
    		isAdjusted = true;
    	}
    	applyOptions();
    }
	
	/** Apply current options */
	public void applyOptions() {
		menuStraight.setState(!isRightAngled);
		menuRightAngled.setState(isRightAngled);
		menuIdentical.setState(!isAdjusted);
		menuAdjusted.setState(isAdjusted);
		
		// options are activated/deactivated by
		// activating/deactivating the corresponding team:
		
		if (isRightAngled)
			_versionA.activate();
		else
			_versionA.deactivate();
		
		if (isAdjusted)
			_versionB.activate();
		else
			_versionB.deactivate();

		// trigger redisplay using new layout
		_view.setHierarchy(_hier);
	}
}
