package gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JComponent;
import companydisplay.CompanyHierarchyDisplay;

/**
 * Application view. Displays a hierarchy
 */
@SuppressWarnings("serial")
public class HierarchyView extends JComponent {
	
	CompanyHierarchyDisplay _hierarchy = null;
	boolean _visible = false;
	
	public HierarchyView() {
		addComponentListener(new MyComponentListener());
	}
	
	/** Get currently displayed hierarchy */
	public CompanyHierarchyDisplay getHierarchy() {
		return _hierarchy;
	}
	
	/** Set a hierarchy to be displayed */
	public void setHierarchy(CompanyHierarchyDisplay hier) {
		_hierarchy = hier;
		_hierarchy.setView(this);
		if (_visible) {
			_hierarchy.calculateLayout();
		}
	}
	
	/** Paint view area */
	public void paint(Graphics g) {
		/* fill background with white color */
		g.setColor(Color.white);
		g.fillRect(0, 0, getWidth(), getHeight());
				
		/* draw hierarchy */
		if (_hierarchy != null) {
			if (!_visible) {
				_hierarchy.calculateLayout();
				_visible = true;
			}
			g.setColor(Color.black);
			_hierarchy.draw(g);
		}
	}
	
	/**
	 * Handle component resize event
	 */
	public class MyComponentListener extends ComponentAdapter {
		
		public void componentResized(ComponentEvent e) {
			if (_hierarchy != null) {
				_hierarchy.calculateLayout();
			}
	    }
	}
}
