package hierarchydisplay;

import java.awt.Component;
import java.awt.Graphics;

/**
 * Hierarchy display collaboration interface.
 */
abstract public team class HierarchyDisplay {
	
	/** expected - get root node */
	public abstract Node getRoot();
	
	/** provided - (re)calculate hiearchy layout */
	abstract public void calculateLayout();
	
	/** provided - refresh display */
	abstract public void refresh();
	
	/** provided - draw hierarchy */
	abstract public void draw(Graphics g);
	
	/** provided - set owner component */ 
	abstract public void setView(Component view);
		
	/** 
	 * Base node 
	 */
	abstract public class Node {
		/** expected - get display text */
		abstract public String getText();
		
		/** provided - check if text fits in node */
		abstract public boolean textFits(String text); 
	}
	
	/**
	 *  Composite node 
	 */
	abstract public class CompositeNode extends Node {
		
		/** expected - get i-th child */
		abstract public Node getChildAt(int i); 
		
		/** expected - get number of children */
		abstract public int getChildCount(); 
	}
}
