package tree;

import java.util.*;
import javax.swing.JTree;
import javax.swing.tree.*;
import javax.swing.JComponent;

/**
 * An Object Teams enabled version of javax.swing.tree.JTree.
 * Helper role NodeEnumeration is defined in this file.
 * Roles in separate files:
 * @role MyTreeModel
 * @role MyTreeNode
 * @role LeafNode
 */
public team class TeamTree {

    protected MyTreeModel model;
    protected MyTreeNode root;
    protected JTree jtree; 

    /** This ctor is needed so that subclasses can bypass the ctor below.  */
    protected TeamTree() {}

    /**
     * Create a tree for the given root node.
     * @param root root note to display in the tree.
     */
    protected TeamTree (MyTreeNode root) {
        activate();
        this.model = new MyTreeModel(root);
        jtree = new JTree(model);
        jtree.setRootVisible(false);    
    }

    /**
     * Get the view component implementing the tree view.
     * @return 
     */
    public JComponent getView() {
        return jtree;
    }
    
    /** For accessing the children of a node. */
    protected class ChildrenEnumeration implements Enumeration<Object> {
        private MyTreeNode node;
        private int pos = 0;
        public ChildrenEnumeration (MyTreeNode node) {
            this.node = node;
        }
        public boolean hasMoreElements () {
            return pos < node.getChildCount();
        }
        public Object nextElement () {
            return node.getChildAt(pos++);
        }
        // be defensive in argument type:
        public int indexOf (TreeNode other) {
            int len = node.getChildCount();
            for (int i = 0; i < len; i++)
                if (node.getChildAt(i) == other) return i;
            return -1;
        }
    }
} 